@@ -22,7 +22,7 @@ Examples:
     1 2 3
     4 5 6
     7 8
-    >>> print_board(shuffle_board(b))
+    >>> print_board(shuffle(b))
     1 6 7
     4   8
     5 3 2
@@ -65,6 +65,7 @@ from slidingpuzzle.board import (
     shuffle,
     shuffle_lazy,
     solution_as_tiles,
+    visit,
 )
 
 from slidingpuzzle.algorithms import (
